#
# This is file for drbl work zone generating
# $Id: do_drbl.pl,v 2.12 2003/10/27 14:48:13 agk Exp $
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   Copyright 2002, Koulik Andrei <agk@agk.nnov.ru>
#



use strict;
use POSIX;

# Code parameters

my $defalutConfig = '/usr/local/etc/drblnode.conf';
my (@mandatoryParams) = ('dig','aggr_net_number');

# global declarations
my %Config;
my ($Id,$Revision) = ('$Id: do_drbl.pl,v 2.12 2003/10/27 14:48:13 agk Exp $','$Revision: 2.12 $');

require '/usr/local/share/drbl/config.pl';
require '/usr/local/share/drbl/drbllib.pl';

cfgDoFile( (defined($ARGV[0])) ? $ARGV[0] : $defalutConfig, \%Config )||
  die "cannot parse the config file\n";
map { die "Error: parameter $_ not found\n" unless defined $Config{$_} }
  @mandatoryParams;

my %sources;
my %masterIP;

keys(%masterIP) = 16384;


#Build work zones
outNot("Build Working zones...");

while( my($zn,$hp) = each(%{$Config{work}}) ) {
  %masterIP=();
  while( my ($srcnick,$shp) = each( %{$hp->{'src'}} )) {
    outDeb("Do source: $srcnick");
    if ( defined $shp -> {'skip'} ) {
      outNot("Skip source $srcnick for zone $zn");
      next;
    }
    my ($zone,$master,$weight,$res,$target,$dig);
    # parse the target ( !!!)
    $res = parseTarget($shp->{'target'},\$zone,\$master,\$weight);
    if ( not $res ) {
      outErr("Bad target (". $shp->{'target'} .") of zone $srcnick");
      die "Fatal error. Exit\n";
    } elsif ( $res == 1 ) {
      ($master,$weight) = ($shp->{'master'},$shp->{'weight'});
    } elsif ( $res == 2 ) {
      $weight = $shp->{'weight'};
    } elsif ( $res == 4 ) {
      $master = $shp->{'master'};
    } elsif ( $res == 5 ) {
      $weight = $shp->{'weight'};
      $target = $zone;
    }
    if ( $res == 5 ) {
      $dig = "<$zone";
      $shp->{'display'} = $srcnick unless defined $shp->{'display'};
    } else {
      $dig = $Config{'dig'}." \@$master $zone axfr|";
      $target = "$zone\@$master";
      $shp->{'display'} = $target unless defined $shp->{'display'};
    }
    outNot("Retrieve zone $srcnick($target)");
    outDeb("with '$dig'");
    $sources{$target} = suck_zone2($dig) if not defined $sources{$target};
    if ( not $sources{$target} ) {
      outErr("Can't retrieve the zone $target");
      die "Fatal error. Exit.\n";
    }
    outNot("Process zone $srcnick($target)");
    foreach my $addr (keys %{$sources{$target}}) {
#      outDeb("Check: $addr");
      unless ($addr =~ /^[0-9]+(\.[0-9]+){0,3}$/) {
	outNot("Rejected: Bad IP: $addr") unless $addr eq '@';
	next;
      }
      next unless $addr =~ /\./ or (defined $Config{'allow_a_class_nets'} );
      addFilter(\%masterIP,$addr,'A',$sources{$target}->{$addr}->{'A'}
		,$weight,$hp->{'threshold'},$srcnick);
      if ( (defined $Config{'keep_source_txt'}) && ($sources{$target}->{$addr}->{'TXT'})) {
	foreach my $txt (keys %{$sources{$target}->{$addr}->{'TXT'}}) {
	  addFilter(\%masterIP,$addr,'TXT',"$shp->{display}:". $txt);
	}
      }
    }
  } #All zone collected

  my (@IPClasses) = ((),(),(),());
  foreach (keys %masterIP) {
    push @{$IPClasses[ tr/\./\./ ]},$_ unless $masterIP{$_}->{'weight'} == 0;
  }
  outDeb("Recost a masked networks");
  for(my $netc = 0,my $sk = 0 ;$netc < 4; $netc++) {
    outDeb("Class No $netc contain:".$#{$IPClasses[$netc]});
    next if $#{$IPClasses[$netc]} < 0;
    $sk++;
    next if $sk < 2;
    foreach my $addr (@{$IPClasses[$netc]}) {
      next unless $masterIP{$addr}->{'dump'} eq 'NO';
      my $un =shiftAddress($addr);
      while( ($un) && (not defined $masterIP{$un}) ) { $un = shiftAddress($un);}
      next unless $un;
      next  unless $masterIP{$un}->{'dump'} eq 'NO';
      while( my($src,$weight) = each( %{$masterIP{$un} ->{src}}) ) {
	addFilter(\%masterIP,$addr,'A','127.0.0.2'
		  ,$weight,$hp->{'threshold'},$src);
	foreach (keys %{$masterIP{$un}->{txt}}) {
	  addFilter(\%masterIP,$addr,'TXT',"$_(U)") if $_;
	}
      }
    }
  }

  my $headerName = cfgGetAltValue('header',$hp,$Config{'defaults'});

  # zone processed, dump
  outNot("Dump zone $zn in file: ". $hp->{'output'});
  if ( not open(ZOUT, ">".$hp->{'output'}) ) {
    outErr("Can't open output file ".$hp->{'output'}." for zone $zn: $!");
    die "Fatal error\n";
  }
  outHeader(\*ZOUT,$headerName) if $headerName;
  outInfoHeader(\*ZOUT);
  #out zone specific parameters
  print ZOUT "\tIN\tTXT\t\"DRBL-Threshold=$hp->{threshold}\"\n";
  while( my($addr,$p) = each(%masterIP) ){
    my $upperNet= shiftAddress($addr);
    next if $p->{'dump'} ne 'OK';
    #look for masking network
    while( ($upperNet)  && ($masterIP{$upperNet}->{'dump'} ne 'OK') ) {
        $upperNet = shiftAddress($upperNet);
    }
    next if $upperNet;

    if ($p->{'right'} ne '') { 
    	print ZOUT  (( ($addr =~ tr/\./\./) == 3)? '':'*.' )
      		. "$addr\tIN\tA\t". $p->{'right'} ."\n";
    	foreach (keys %{$p->{'txt'}})
      		{print ZOUT "\tIN\tTXT\t\"$_\"\n" };
    }
  }
  close(ZOUT);
}

#cfgOut('',\%Config);

sub addFilter {
  my ($pAddrs,$addr,$aType,$val, $weight,$treshold,$FSrc) = @_;
  my $pAddr; # for optimization
  $addr =~ s/^.*\*\.//;
  $pAddrs -> {$addr} = &newAddrEnt() if not defined  $pAddrs -> {$addr};
  $pAddr = $pAddrs -> {$addr};
  if ( $aType eq 'TXT' ) {
    $pAddr -> {'txt'} -> {$val} = '' if $val;
    return;
  }
  return if defined $pAddr->{'src'}->{$FSrc};
  $pAddr->{'src'}->{$FSrc} = $weight;
  return if $pAddr->{'dump'} eq 'OK';
  $pAddr->{'weight'} += $weight;
  if ($pAddr->{'weight'} >= $treshold ) {
    $pAddr->{'right'} = $val;
    approveNet($pAddrs,$addr);
  }
}
sub approveNet {
  my ($aStore,$net) = @_;
  $aStore->{$net} -> {'dump'} = 'OK';
  return unless $net = shiftAddress($net);
  $aStore->{$net} =  &newAddrEnt() if not defined  $aStore->{$net};
  return if $aStore->{$net} -> {'dump'} eq 'OK';
  return unless  ++$aStore->{$net} -> {'netcount'} >= $Config{'aggr_net_number'};
  outNot("Aggrigating network added: $net");
  approveNet($aStore,$net);
  $aStore->{$net} -> {'txt'} -> {'Aggregated lower classes'} = '';
}

sub newAddrEnt {
  return  {
	   'right' => '127.0.0.2',
	   'weight' => 0,
	   'netcount' => 0,
	   'dump' => 'NO',
# using hash is bit slower but need to remove duplicates
	   'txt' => {},
	   'src' => {},
	  };

}

sub shiftAddress {
  my ($n) = @_;
  return unless  $n =~ /\./; 
  $n =~ s/^[0-9]+\.//;
  return $n;
}

sub outInfoHeader {
  my ($ostream) = @_;
  print $ostream "\tIN\tTXT\t\"DRBL-Version=$Revision\"\n";
  print $ostream "\tIN\tTXT\t\"DRBL-Type=WORK\"\n";
  print $ostream "\tIN\tTXT\t\"DRBL-Info=" . &cfgGetValue(\%Config,'info.target')."\"\n";
  my $Date = ctime(time);
  chomp($Date);
  print $ostream "\tIN\tTXT\t\"DRBL-Date=$Date\"\n";
}
