#
# This is config parse module
# $Id: /usr/local/share/drbl/config.pl,v 2.5 2003/10/28 06:39:39 agk Exp $
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   Copyright 2002, Koulik Andrei <agk@agk.nnov.ru>
# 

use strict;

# re of value delemiter
my $cfgDelim='\s*=\s*';
# printable value of delemiter
my $cfgPDelim='=';
#re of path delemiter
my $cfgPathDelim='\.';
# printable value of path delemiter;
my $cfgPathPDelim= '.';
# prefix of config parser messages
my $cfgLogPrefix='Config:';
# out a message

sub cfgLogMsg {
  my($msg) = join('\n',@_);
  my (@logParts)=split(/[\n\r]+/,$msg);
  map { print STDERR "$cfgLogPrefix $_\n"; } @logParts;
}

#parse file
# arg 1 - filename
# 2 - ref to result hash
# returns 1 if succefuly parsed
#
sub cfgDoFile {
  my($fileName,$cfgPtr) = @_;
  my($lv,$rv);
  my($cnt) = 0;
  if ( open(CfgFD,"<$fileName")) { 
    while(<CfgFD>) {
      chomp;
      $cnt++;
      # remove all comments (do not check for "" literals :( )
      s/\#.*$//;
      #revove leading whites
      s/^\s+//;
      s/\s+$//;
      # skip an empty line
      next if /^$/;
      ($lv,$rv) = split(/$cfgDelim/,$_);
      if ($rv eq '') {
	cfgLogMsg("Syntax error in $fileName:$cnt");
	return;
      }
      cfgSetValue($cfgPtr,$lv,$rv);
    }
    close(CfgFD);
  } else {
    cfgLogMsg("Can't open file: $fileName");
    return;
  }
  return "1";
}

# indert value into hash table
# args:
# 1 - pointer to hash
# 2 - path to value
# 3 - the value

sub cfgSetValue {
  my($hashBase,$path,$val)=@_;
  my(@pathParts) = split(/$cfgPathDelim/,$path);
  my($firstStep) = lc shift @pathParts;

  if ( $#pathParts >=0 ) {
    $hashBase->{$firstStep} = {} if not defined $hashBase->{$firstStep};
    cfgSetValue($hashBase->{$firstStep},join($cfgPathPDelim,@pathParts),$val);
  } else {
    $hashBase->{$firstStep} = $val;
  }
}
sub cfgGetValue {
  my($hashBase,$param) = @_;
  my(@pathParts) = split(/$cfgPathDelim/,$param);
  my($firstStep) = lc shift @pathParts;
  if ( $#pathParts >=0 ) {
    return  if not defined $hashBase->{$firstStep};
    return  cfgGetValue($hashBase->{$firstStep},join($cfgPathPDelim,@pathParts));
  } else {
    return $hashBase->{$firstStep} if defined  $hashBase->{$firstStep};
  }
  return;
}

sub cfgGetAltValue {
  my($param,@hashBases) = @_;
  my $val;
  foreach my $ch (@hashBases) {
    $val = cfgGetValue($ch,$param);
    return $val if $val;
  }
  return;
}

#
# out config stored in hash to file
# args
# 1 - filename
# 2 - pointer to hash
sub cfgOut {
  my($fileName,$cfgPtr) = @_;
  cfgOutBranch(\*STDOUT,$cfgPtr,'');
}


sub cfgOutBranch {
  my($ostream, $hashBase, $prefix) = @_;
  foreach my $e (keys %$hashBase) {
    if ( ref($hashBase->{$e}) eq 'HASH' ) {
      cfgOutBranch($ostream,$hashBase->{$e},"$prefix$e$cfgPathPDelim");
    } else {
      print  $ostream "$prefix$e$cfgPDelim" . $hashBase -> {$e} ."\n";
    }
  }
}
