#
# This is file from drbl zone generating
# $Id: /usr/local/share/drbl/drbllib.pl,v 2.13 2003/10/28 06:54:30 agk Exp $
# You may copy this module under the terms of the GNU General Public License 
# 
# Copyright 2002, Koulik Andrei <agk@agk.nnov.ru>
#
# Used elements of build_drbl v1.06 writen by:
#
# Cyril A. Vechera cyril@piter.net 2000.05.12
# Dmitry Morozovsky <marck@rinet.ru>
# Andy Igoshin <ai@vsu.ru>
# Andrey V. Stolyarov (crocodil@croco.net)
# D Kelmi <miksir@sub.ru>
# Vladimir B. Grebenschikov <vova@express.ru>
#

use strict;

# if you need dates marks while output
# set
# my $needDate = 1;
my $needDate = undef;


# URL symbols set
my $USS = '[a-zA-Z\-\.0-9]+';
# Weight mask
my $WSS = '(\d+|(\d)?\.\d+)?';

#display errors then empty rr data found
my $showEmptyDataErrors = 1;

my $fltAsTl = {'A'=>'SCALAR','TXT'=>'HASH'};
my $fltTa   = {'TXT'=>'ARRAY'};

my ($outPref,$outStream,$outLevel)=('DRBL',\*STDOUT,10);

sub initOutput {
  ($outPref,$outStream,$outLevel) = @_;
}

sub setNeedDate {
    my $was = $needDate;
    $needDate = shift @_;
    return $was;
}

sub outNot {
  return if $outLevel <5;
  print $outStream &mkGenMsg(&getDate . "$outPref(Notice)",@_);
}
sub outDeb {
  return if $outLevel <10;
  print $outStream &mkGenMsg(&getDate . "$outPref(Debug)",@_);
}
sub outMsg {
  print $outStream &mkGenMsg(&getDate . $outPref,@_);
}
sub outErr {
  print STDERR &mkGenMsg(&getDate . "$outPref(ERROR)",@_);
}

sub getDate {
    return ($needDate)?localtime()." ":"";
}

# 
# Create splited message with specific prefix
#
sub mkGenMsg {
  my $outMsg;
  my($pref,@imsg) = @_;
  my (@logParts)=split(/[\n\r]+/, join('\n',@imsg) );
  map { $outMsg .= "$pref:\t$_\n"; } @logParts;
  return $outMsg;
}


#############################################################
## Service functions
#############################################################

# Praser target, format: zone[@master][/weight]
# args in
# 1 - target
# out
# 2 - zone
# 3 - master
# 4 - weight
# returns
# 1 if zone
# 2 if zone@master
# 3 if zone@master/weight
# 4 if zone/weight
# 5 has leading '/' - seems the path name
# undef if error

sub parseTarget {
  my ($target,$pZone,$pMaster,$pWeight) = @_;
  if ( $target =~ /^($USS)\@($USS)\/($WSS)$/ ) {
    $$pZone = $1;
    $$pMaster = $2;
    $$pWeight = $3;
    return "3";
  } elsif ( $target =~ /^($USS)\@($USS)$/ ) {
    $$pZone = $1;
    $$pMaster = $2;
    $$pWeight = undef;
    return "2";
  } elsif ( $target =~ /^($USS)\/($WSS)$/ ) {
    $$pZone = $1;
    $$pMaster = undef;
    $$pWeight = $3;
    return "4";
  } elsif ( $target =~ /^($USS)$/ ) {
    $$pZone = $1;
    $$pMaster = undef;
    $$pWeight = undef;
    return "1";
  } elsif ( $target =~ /^[\/\.].+/ ) {
    $$pZone = $target;
    $$pMaster = undef;
    $$pWeight = undef;
    return "5";
  }
  return;
}


 sub suck_zone {
        my ($source) = @_;
        my ($table, $lname, $name);
        my ($ttl, $class, $type, @data, $data);

        if ( not open (AXFR,$source) ) {
	  outErr("Can't open zone source");
	  return;
	}
        $table = {};

        while (<AXFR>) {
                next if /^;/;
                $name = ''; chomp;
                if ( /^\s/ && $lname) { # ommit name
                        ($ttl, $class, $type, @data) = split;
                        if ($ttl eq 'IN') { # record without TTL
                                unshift @data, $type;
                                $type = $class;
                                $class = $ttl;
                        }
                        $name = $lname;
                } elsif ( /^(\d+|\*)/ ) {
                        ($name, $ttl, $class, $type, @data) = split;
                        if ($ttl eq 'IN') { # record without TTL
                                unshift @data, $type;
                                $type = $class;
                                $class = $ttl;
                                $ttl = '1D';    # really unneded
                        }
                        $lname = $name;
                } else {
                        $lname = '';
                }
                next if (!$name || ($type ne 'A' && $type ne 'TXT'));
                $name =~ s/[a-zA-Z\.]*$//g;
                $name =~ s/^\*\.//o;
                $data = join(';', @data);
                $data =~ s/\"//g;
                ${$table}{$name} = {} if (!defined(${$table}{$name}));
                ${$table}{$name}->{'A'} = $data if ($type eq 'A');
                ${$table}{$name}->{'TXT'} .= $data  if ($type eq 'TXT' && (${$table}{$name}->{'TXT'} !~ /\Q$data\E\n/));
        }

        close (AXFR);

        return $table;
}

sub outHeader {
  my ($ostream,$fName) = @_;

  if( not open(FOUT,"<$fName")) {
    outErr("can't open file $fName: $!");
    return undef;
  }
  while(<FOUT>) {
    s/\$time\$/time()/ge;
    print $ostream "$_";
  }
  close(FOUT);
  return '1';
}

sub suck_zone2 {
  my($source) = @_;
  return suck_zone_u($source,$fltAsTl);
}


sub suck_zone1 {
  my($source,$base) = @_;
  return suck_zone_u($source,$fltTa,$base);
}

sub suck_zone_u {
  my($source,$filter,$Base) = @_;
  my $table = {};
  my $name = '@';
  my ($Origin) = '';
  my ($namec,$namec_save,$type,@Data,$Data);
  my ($statHosts)=(0);
  my $baseLock = (defined $Base)?1:undef;

  my $localDebug = undef;
#  $localDebug = 1;
#  outDeb("Filter: ".join(',',keys %$filter));
  if (defined $Base) {
    $Base = "$Base." unless $Base =~ /\.$/;
    $Base = ".$Base" unless $Base =~ /^\./;
  }
  outDeb("exec zone supplier: $source, Base: $Base");
  if ( not open (AXFR,$source) ) {
    outErr("Can't open zone source $source");
    return;
  }
  while (<AXFR>) {
    next if /^\s*;/;
    next if /^\s*$/;
    print ">$_\n" if $localDebug;
    if (/^\s/) {
      (@Data) = split;
    } else {
      $namec_save = $namec; #save to restore after some directives ($TTL)
      ($namec,@Data) = split;
    }
    #check for derectives
    if ($namec =~ /^\$/) {
      #process ORIGIN
      if ( $namec eq '$ORIGIN' ) {
	$Origin = ($Data[0] =~ /^\./)? $Data[0]:".$Data[0]";
#	$Origin = ($Data[0] =~ /^\./)? "":".$Data[0]";
	print "Set origin: $Origin\n" if $localDebug;
#	print "Set base: $Origin\n" unless  defined $Base;
	$Base = $Origin unless defined $Base;
	$namec = ""; $namec_save = "";#reset  
	next;
      }
      $namec= $namec_save; #restore value
      next;
    }
    $type = uc(shift @Data);
    #
    # Sometime the tag 'IN' omitted in a zone
    # file. It is bad but we must be regarded
    #
    outDeb("Type 1: <$type>") if $localDebug;
    unless ($type eq 'SOA' or defined  $filter->{$type}) { # with out IN
      $type = uc(shift @Data);
      outDeb("Type 2: <$type>") if $localDebug;
      next if $type =~ /^;/; #check against faked record
      unless ($type eq 'SOA' or defined  $filter->{$type}) { # IN only
	$type = uc(shift @Data);
	outDeb("Type 3: <$type>") if $localDebug;
	next if $type =~ /^;/;
	next unless ($type eq 'SOA' or defined  $filter->{$type}) #time and IN
      }
    }
    outDeb("Type passed: $type, Base: <$Base>, Origin: <$Origin> ") if $localDebug;
    if ($type eq 'SOA') {
      next if $baseLock;
      $Base = ($namec eq '@')?$Origin:".$namec$Origin";
      $namec="\@.$namec" unless $namec eq '@';
      outDeb("Set base: $Base") if $localDebug;
      next;
    }
    $name = $namec . $Origin;
    $name =~ s/\Q$Base\E$//;
    $name =~ s/^\*\.//;
    $name = '@' if $name eq "" or ".$name" eq $Base;
    outDeb("Type <$type>, Name: <$name>, Base: <$Base>, Origin: <$Origin>") if $localDebug;
#    outDeb("passed");

    $Data = join ' ',@Data;
    $Data =~ s/\"//g;
    if ( $Data =~ /^\s*$/) {
      outErr("Emty data section: row skipped (name: $name, type: $type)") if $showEmptyDataErrors;
      next;
    }

    unless (exists $table -> {$name}) {
      $table -> {$name} = {};
      $statHosts++;
    }
    if ( $filter->{$type} eq 'ARRAY' ) {
      $table->{$name}->{$type} = [] unless exists  $table->{$name}->{$type};
      push @{$table->{$name}->{$type}},$Data;
    }elsif ( $filter->{$type} eq 'HASH' ) {
      $table->{$name}->{$type} = {} unless exists  $table->{$name}->{$type};
      $table->{$name}->{$type}->{"$Data"} = "";
    } elsif( $filter->{$type} eq 'SCALAR') {
#      outDeb("set $Data to $name");
      $table->{$name}->{$type} = $Data;
    } else {
      $table->{$name}->{$type} .= " $Data";
    }
  }
  close AXFR;
  outDeb("Loaded: Hosts: $statHosts");
  return $table;
}
